@@ -48,7 +48,7 @@ namespace Beagle.Daemon.BlamQueryable {
 
 		int wdBlam = -1;
 
-		public BlamQueryable () : base (Path.Combine (PathFinder.RootDir, "BlamIndex"))
+		public BlamQueryable () : base ("BlamIndex")
 		{
 			blamDir = Path.Combine (Path.Combine (Environment.GetEnvironmentVariable ("HOME"), ".gnome2"), "blam");
 		}
@@ -74,8 +74,7 @@ namespace Beagle.Daemon.BlamQueryable {
 
 			base.Start ();
 
-			Thread th = new Thread (new ThreadStart (StartWorker));
-			th.Start ();
+			ExceptionHandlingThread.Start (new ThreadStart (StartWorker));
 		}
 
 		private void OnInotifyEvent (int wd,
@@ -98,7 +97,7 @@ namespace Beagle.Daemon.BlamQueryable {
 		{
 			FileInfo file = new FileInfo (Path.Combine (blamDir, "collection.xml"));
 
-			if (Driver.IsUpToDate (file.FullName))
+			if (this.FileAttributesStore.IsUpToDate (file.FullName))
 				return;
 
 			ChannelCollection collection = null;
